@@ -27,6 +27,7 @@
 #include <cassert>
 #include <iostream>
 #include <array>
+#include <algorithm>
 
 #include <ngtcp2/ngtcp2_crypto.h>
 
@@ -77,43 +78,53 @@ int generate_secret(uint8_t *secret, size_t secretlen) {
   return 0;
 }
 
-std::optional<std::string> read_token(const std::string_view &filename) {
+namespace {
+void openssl_free_wrap(void *ptr) { OPENSSL_free(ptr); }
+} // namespace
+
+std::optional<std::string> read_pem(const std::string_view &filename,
+                                    const std::string_view &name,
+                                    const std::string_view &type) {
   auto f = BIO_new_file(filename.data(), "r");
   if (f == nullptr) {
-    std::cerr << "Could not open token file " << filename << std::endl;
+    std::cerr << "Could not open " << name << " file " << filename << std::endl;
     return {};
   }
 
   auto f_d = defer(BIO_free, f);
 
-  char *name, *header;
+  char *pem_type, *header;
   unsigned char *data;
   long datalen;
-  std::string token;
-  if (PEM_read_bio(f, &name, &header, &data, &datalen) != 1) {
-    std::cerr << "Could not read token file " << filename << std::endl;
+
+  if (PEM_read_bio(f, &pem_type, &header, &data, &datalen) != 1) {
+    std::cerr << "Could not read " << name << " file " << filename << std::endl;
     return {};
   }
 
-  OPENSSL_free(name);
-  OPENSSL_free(header);
+  auto pem_type_d = defer(openssl_free_wrap, pem_type);
+  auto pem_header = defer(openssl_free_wrap, header);
+  auto data_d = defer(openssl_free_wrap, data);
 
-  auto res = std::string{data, data + datalen};
+  if (type != pem_type) {
+    std::cerr << name << " file " << filename << " contains unexpected type"
+              << std::endl;
+    return {};
+  }
 
-  OPENSSL_free(data);
-
-  return res;
+  return std::string{data, data + datalen};
 }
 
-int write_token(const std::string_view &filename, const uint8_t *token,
-                size_t tokenlen) {
+int write_pem(const std::string_view &filename, const std::string_view &name,
+              const std::string_view &type, const uint8_t *data,
+              size_t datalen) {
   auto f = BIO_new_file(filename.data(), "w");
   if (f == nullptr) {
-    std::cerr << "Could not write token in " << filename << std::endl;
+    std::cerr << "Could not write " << name << " in " << filename << std::endl;
     return -1;
   }
 
-  PEM_write_bio(f, "QUIC TOKEN", "", token, tokenlen);
+  PEM_write_bio(f, type.data(), "", data, datalen);
   BIO_free(f);
 
   return 0;
